@@ -1,8 +1,8 @@
-import time
 import datetime
 import functools
-import sys
 import inspect
+import sys
+import time
 
 from dateutil import parser
 
@@ -10,6 +10,10 @@ real_time = time.time
 real_date = datetime.date
 real_datetime = datetime.datetime
 
+try:
+    import copy_reg as copyreg
+except ImportError:
+    import copyreg
 
 # Stolen from six
 def with_metaclass(meta, *bases):
@@ -154,6 +158,27 @@ def convert_to_timezone_naive(time_to_freeze):
     return time_to_freeze
 
 
+def pickle_fake_date(datetime_):
+    # A pickle function for FakeDate
+    return FakeDate, (datetime_.year,
+        datetime_.month,
+        datetime_.day,
+    )
+
+
+def pickle_fake_datetime(datetime_):
+    # A pickle function for FakeDatetime
+    return FakeDatetime, (datetime_.year,
+        datetime_.month,
+        datetime_.day,
+        datetime_.hour,
+        datetime_.minute,
+        datetime_.second,
+        datetime_.microsecond,
+        datetime_.tzinfo,
+    )
+
+
 class _freeze_time(object):
 
     def __init__(self, time_to_freeze_str, tz_offset, ignore):
@@ -171,14 +196,18 @@ class _freeze_time(object):
 
     def decorate_class(self, klass):
         for attr in dir(klass):
-            if not attr.startswith("test"):
+            if attr.startswith("_"):
                 continue
 
             attr_value = getattr(klass, attr)
             if not hasattr(attr_value, "__call__"):
                 continue
 
-            setattr(klass, attr, self(attr_value))
+            try:
+                setattr(klass, attr, self(attr_value))
+            except TypeError:
+                # Sometimes we can't set this for built-in types
+                continue
         return klass
 
     def __enter__(self):
@@ -188,13 +217,18 @@ class _freeze_time(object):
         self.stop()
 
     def start(self):
+        # Change the modules
         datetime.datetime = FakeDatetime
         datetime.date = FakeDate
         fake_time = FakeTime(self.time_to_freeze, time.time)
         time.time = fake_time
 
+        copyreg.dispatch_table[real_datetime] = pickle_fake_datetime
+        copyreg.dispatch_table[real_date] = pickle_fake_date
+
+        # Change any place where the module had already been imported
         for mod_name, module in list(sys.modules.items()):
-            if module is None:
+            if mod_name is None or module is None:
                 continue
             if mod_name.startswith(tuple(self.ignore)):
                 continue
@@ -219,14 +253,18 @@ class _freeze_time(object):
         datetime.datetime.tz_offsets.pop()
         if not datetime.datetime.times_to_freeze:
             datetime.datetime = real_datetime
+            copyreg.dispatch_table.pop(real_datetime)
 
         datetime.date.dates_to_freeze.pop()
         if not datetime.date.dates_to_freeze:
             datetime.date = real_date
+            copyreg.dispatch_table.pop(real_date)
 
         time.time = time.time.previous_time_function
 
         for mod_name, module in list(sys.modules.items()):
+            if mod_name is None or module is None:
+                continue
             if mod_name.startswith(tuple(self.ignore)):
                 continue
             if mod_name != 'datetime':
